/* hf_creation_dialog.c - height field creation interface functions
 *
 * Copyright (C) 2001, 2006 Patrice St-Gelais
 *         patrstg@users.sourceforge.net
 *         http://geomorph.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "hf_creation_dialog.h"
#include "render.h"
#include "hf.h"
#include "hf_wrapper.h"

#define NBSIZES 6
void size_callb (gpointer data,GtkWidget *wdg);

omenu_list_type sizes_list[NBSIZES] = {
	NULL,128,NULL,"128",size_callb,
	NULL,256,NULL,"256",size_callb,
	NULL,512,NULL,"512",size_callb,
	NULL,1024,NULL,"1024",size_callb,
	NULL,2048,NULL,"2048",size_callb,
	NULL,4096,NULL,"4096",size_callb,
};

// Toolbar icons for HF creation algorithm
#define NBHFTYPES 4
#include "../icons/uniform.xpm"
#include "../icons/subdiv1.xpm"
#include "../icons/subdiv2.xpm"
// #include "../icons/waves.xpm"
// #include "../icons/waves1.xpm"
#include "../icons/surfadd.xpm"


command_item_struct hftypes[NBHFTYPES] = {
"Type", "Uniform", "Uniform color",0, (gchar **) uniform_xpm, GDK_LEFT_PTR, hfuniform_callb, NULL, NULL,TRUE,
"Type", "Subdiv1", "Subdivision 1",0, (gchar **) subdiv1_xpm, GDK_LEFT_PTR, hfsubdiv1_callb,NULL, NULL,FALSE,
"Type", "Subdiv2", "Subdivision 2",0, (gchar **) subdiv2_xpm, GDK_LEFT_PTR, hfsubdiv2_callb, NULL, NULL,FALSE,
"Type", "SurfSum", "Progressive sum of surfaces",0, (gchar **) surfadd_xpm, GDK_LEFT_PTR, hfsurfadd_callb, NULL, NULL,FALSE
};

/***************************************************************************/

gint get_display_scale (gint hf_size) {
//	Guess display scale from HF size and current display size...
//	or take the value given in the RC file
	static gint def_size = 0;
	if (DOC_DISPLAY_SIZE) {
		if (hf_size>DOC_DISPLAY_SIZE)
			return log2i(hf_size / DOC_DISPLAY_SIZE);
		else 
			return 0;
	}
	else {
		if (!def_size)
			def_size =  hf_default_size();
		if (hf_size > def_size)
			return log2i(hf_size / def_size);
		else
			return 0;
	}
}

void size_callb (gpointer data,GtkWidget *wdg) {
	// Sets size when creating a new HF
	gint hf_size;
	static gchar area_label[12];
	omenu_struct_type *opt_menu;
	hf_wrapper_struct *hfw;
	opt_menu = (omenu_struct_type *) data;
	hfw = (hf_wrapper_struct *) * (hf_wrapper_struct **) opt_menu->data;

// printf("HFWrapper adr in SIZE_CALLB: %d;  HFWrapper: %d\n",opt_menu->data,hfw);

	hf_size = opt_menu->list[opt_menu->current_index].num;
	if (!hfw)
		return;
	if (hf_size == hfw->hf_struct->max_x)
		//	No change!
		return;

	sprintf(area_label, "%d x %d", hf_size, hf_size);
	gtk_frame_set_label(GTK_FRAME(hfw->draw_area_frame), area_label);

	hf_free(hfw->hf_struct);
	hfw->hf_options->hf_size = hf_size;
	hfw->display_scale = get_display_scale(hf_size);
	hfw->hf_struct = (hf_struct_type *) hf_new(hf_size, hfw->display_scale);
	set_widget_display_scale(hfw);
	hfw->area_size = RIGHT_SHIFT(hf_size,hfw->display_scale);
	hfw->size_change_request = TRUE;
	
	gtk_widget_set_size_request (hfw->draw_area_frame, hfw->area_size+SCALE_PAD, hfw->area_size+SCALE_PAD);

// printf("HF_SIZE: %d; DISPLAY_SCALE: %d; AREA_SIZE: %d\n",hf_size, hfw->display_scale, hfw->area_size);
	gl_update_details_display (hfw->gl_preview, hf_size);
	gtk_drawing_area_size(GTK_DRAWING_AREA(hfw->area), 
		hfw->area_size, hfw->area_size);
}

GtkWidget *hf_types_toolbar_new(GtkWidget *window, GtkTooltips *tooltips, 
		GtkWidget** tb_widget, gpointer data) {
	GtkWidget *hbox;
	GtkWidget *tb;
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(GTK_WIDGET(hbox));
	define_label_in_box("Type", hbox, FALSE, FALSE, DEF_PAD*0.5);
	tb = standard_toolbar_new(NBHFTYPES,
		hftypes,
		tooltips,
		window,
		data,
		GTK_ORIENTATION_HORIZONTAL,
		GTK_TOOLBAR_ICONS,
		TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), tb, FALSE, FALSE, DEF_PAD*0.5);
	if (tb_widget)
		*tb_widget = tb;
	return hbox;
}

GtkWidget * hf_creation_dialog_new(GtkWidget *window, GtkTooltips *tooltips, 
		gpointer data, gpointer widget_list_ptr) {
	// Initializes the dialog for creating height fields
	// Four parts, from top to bottom:
	//	1. Kind / type of HF
	//	2. Shape filters
	//	3. Generic options (size...)
	//	4. Options for the current HF
	// Returns a vbox container, allowing to add / substract particular controls
	// "window" only needed for heritance of characteristics in child toolbar(s)
	// "tooltips" needed for packing tooltips, could have only one for the whole app.
	// gpointer data:  is normally the address of the current hf_wrapper pointer 
	// (hf_wrapper_struct **)

	GtkWidget *dialog, *hbox, *vbox, *frame;
	gint default_hf_size;
	omenu_struct_type *opt_menu;
	hf_creation_wdg_struct *wdg_str;

	wdg_str = (hf_creation_wdg_struct *) malloc(sizeof(hf_creation_wdg_struct));

	*((hf_creation_wdg_struct **) widget_list_ptr) = wdg_str;

	frame = options_frame_new("HeightField");

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);

	//	1st generic option:  HF size - a power of 2
	//	Default:  some % of screen width
	
	hbox = gtk_hbox_new(FALSE, DEF_PAD);
	gtk_widget_show(hbox);
	define_label_in_box("Size", hbox, FALSE, FALSE, DEF_PAD);

	default_hf_size = hf_default_size();
// printf("HFW_ADR: %d;  HFW: %d;  DEFAULT_HF_SIZE: %d\n",data, *((hf_wrapper_struct**) data),default_hf_size);
	
	create_omenu_with_callb_data (&opt_menu, sizes_list,
		NBSIZES, 
		find_omenu_num(sizes_list, default_hf_size, NBSIZES),
		data);
	gtk_box_pack_start(GTK_BOX(hbox), opt_menu->omenu_ptr, FALSE, FALSE, 0);

	wdg_str->size_menu = opt_menu;

 	gtk_box_pack_start (GTK_BOX (hbox), create_render_bar (window, tooltips, data), FALSE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(vbox),
		hf_types_toolbar_new(window, tooltips, &wdg_str->hf_types_toolbar, data) ,
		FALSE, FALSE, 0);

	//	Other generic options ?

	gtk_container_add(GTK_CONTAINER(frame), vbox);
	dialog = gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(dialog), frame, FALSE, FALSE, 0);
	gtk_widget_show(dialog);

	return dialog;
}

void hf_commit_creation (gpointer data) {
//	Commit the filter
	filter_dialog_struct *fds;
	hf_wrapper_struct *hfw;
	hfw = (hf_wrapper_struct *) data;
// printf("HFW, HFW->creation_mode in hf_commit_creation: %d, %d\n",data,hfw->creation_mode);
// printf("HFW->window->window: %d->%d\n",hfw->window,hfw->window->window);
	hfw->creation_mode = FALSE;  // Supposed to be done in hf_wrapper_copy...
//	hfw->hf_options->current_calculation = NULL;
	hfw->apply_filter = FALSE;

//	Backup the filter definition and nullify it - we 1st commit the HF without the filter
	if (hfw->hf_options->fd_struct) {
		set_filter_defaults (hfw->hf_options->fd_struct);
	}
	
	record_action(hfw, hfw->hf_options->last_creation_action);
	if (hfw->hf_options->fd_struct) {
		fds = hfw->hf_options->fd_struct;
		if (fds->current_filter) {
		    filter_apply(hfw->hf_struct,
			fds->current_filter,
			hfw->hf_options->dist_matrix,
			fds->revert_filter,
			fds->filter_level,
			fds->merge_oper);
//	Now we commit the HF with the filter
		record_action(hfw,_("Shape filter"));
		}
	}
//	If we are in the shape filter subdialog in the tools dialog:
	if (hfw->hf_options->img)
		hfw->hf_options->fd_struct = hfw->hf_options->img->fd_struct;
	else
		hfw->hf_options->fd_struct = NULL;

//	"expose_event" seems required to display the cursor ?
	emit_expose_event(hfw->area);
}

// HF creation

void hfuniform_callb(GtkWidget *wdg, gpointer data) {
	// Display / connect callback for uniform hf (a simple scale from 0 to 0xFFFF)

	hf_wrapper_struct *hfw;

	if (wdg)
		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdg)))
			return;
	hfw = (hf_wrapper_struct *) * (hf_wrapper_struct **) data;

// printf("IN hfuniform_callb:  data: %d;  hfw->hf_options: %d\n", data, hfw->hf_options);

	if (!hfw)
		return;
	//  Options dialog
	if (!hfw->hf_options->uniform_dialog) {
		// NB hfw->hf_options->uniform_value allocated in hf_options_new()
		hfw->hf_options->uniform_dialog = uniform_dialog_new(data,
			*hfw->hf_options->uniform_value);
		gtk_container_add(GTK_CONTAINER(hfw->creation_container),
			hfw->hf_options->uniform_dialog);
	}
	if (hfw->hf_options->current_dialog)
		gtk_widget_hide(hfw->hf_options->current_dialog);
	gtk_widget_show(hfw->hf_options->uniform_dialog);
	hfw->hf_options->last_creation_action = _("Uniform");
	hfw->hf_options->current_dialog = hfw->hf_options->uniform_dialog;
	hfw->hf_options->current_calculation = (gpointer) uniform;
	hfw->hf_options->current_callb = hfuniform_callb;
	hfw->hf_options->current_options = (gpointer) hfw->hf_options->uniform_value;
	hfw->hf_options->flag_uniform_done = FALSE;
	gener_hf(hfw);
}

void hfsubdiv1_callb(GtkWidget *wdg, gpointer data) {
	// Display / connect callback
	// Simple subdivision, square / diamond dots

	hf_wrapper_struct *hfw;
	if (wdg)
		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdg)))
			return;
	hfw = (hf_wrapper_struct *) * (hf_wrapper_struct **) data;
	if (!hfw)
		return;

	//  Options dialog - we create it at the 1st pass
	if (!hfw->hf_options->subdiv1_dialog) {
	//	We create the struct which will receive the data
		hfw->hf_options->subdiv1_options = subdiv1_opt_new();
	//	Dialog creation (with some defaults from subdiv1_opt_new())
		hfw->hf_options->subdiv1_dialog = subdiv1_dialog_new(data);
		gtk_container_add(GTK_CONTAINER(hfw->creation_container),
			hfw->hf_options->subdiv1_dialog);
	}
	if (hfw->hf_options->current_dialog)
		gtk_widget_hide(hfw->hf_options->current_dialog);
	gtk_widget_show(hfw->hf_options->subdiv1_dialog);
	hfw->hf_options->last_creation_action = _("Subdiv1");
	hfw->hf_options->current_dialog= hfw->hf_options->subdiv1_dialog;
	hfw->hf_options->current_calculation = (gpointer) subdiv1;
	hfw->hf_options->current_callb = hfsubdiv1_callb;
	hfw->hf_options->current_options = (gpointer) hfw->hf_options->subdiv1_options;
	gener_hf(hfw);
}

void hfsubdiv2_callb(GtkWidget *wdg, gpointer data) {
	// Display / connect callback
	// Simple subdivision, square / diamond dots - TEST

	hf_wrapper_struct *hfw;
	static gint frq = 50;
	if (wdg)
		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdg)))
			return;
	hfw = (hf_wrapper_struct *) * (hf_wrapper_struct **) data;
	if (!hfw) 
		return;
	//  Options dialog - we create it at the 1st pass
	if (!hfw->hf_options->subdiv2_dialog) {
	//	We create the struct which will receive the data
		hfw->hf_options->subdiv2_options = subdiv2_opt_new(&frq,1);
	//	Dialog creation (with some defaults from subdiv2_opt_new())
		hfw->hf_options->subdiv2_dialog = subdiv2_dialog_new(data);
		gtk_container_add(GTK_CONTAINER(hfw->creation_container),
			hfw->hf_options->subdiv2_dialog);
	}
	if (hfw->hf_options->current_dialog)
		gtk_widget_hide(hfw->hf_options->current_dialog);
	gtk_widget_show(hfw->hf_options->subdiv2_dialog);
	hfw->hf_options->last_creation_action = _("Subdiv2");
	hfw->hf_options->current_dialog = hfw->hf_options->subdiv2_dialog;
	hfw->hf_options->current_calculation = (gpointer) subdiv2;
	hfw->hf_options->current_callb = hfsubdiv2_callb;
	hfw->hf_options->current_options = (gpointer) hfw->hf_options->subdiv2_options;
	gener_hf(hfw);
}

void hfsurfadd_callb(GtkWidget *wdg, gpointer data) {
	// Progressive addition of surfaces

	hf_wrapper_struct *hfw;
	hf_type value;
	if (wdg)
		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdg)))
			return;
	hfw = (hf_wrapper_struct *) * (hf_wrapper_struct **) data;
	if (!hfw)
		return;
	//  Options dialog - we create it at the 1st pass
	if (!hfw->hf_options->surfadd_dialog) {
	//	We create the struct which will receive the data
		hfw->hf_options->surfadd_options = surfadd_opt_new();
		hfw->hf_options->surfadd_options->dist_matrix = hfw->hf_options->dist_matrix;
	//	Dialog creation (with some defaults from surfadd_opt_new())
		hfw->hf_options->surfadd_dialog = surfadd_dialog_new(data);
		hfw->hf_options->surfadd_options->current_filter =
			&((hf_wrapper_struct *)hfw->hf_options->primit_surf)->hf_options->fd_struct->current_filter;
		hfw->hf_options->surfadd_options->revert_filter =
			&((hf_wrapper_struct *)hfw->hf_options->primit_surf)->hf_options->fd_struct->revert_filter;
		hfw->hf_options->surfadd_options->filter_merge =
			&((hf_wrapper_struct *)hfw->hf_options->primit_surf)->hf_options->fd_struct->merge_oper;
		hfw->hf_options->surfadd_options->filter_level =
			&((hf_wrapper_struct *)hfw->hf_options->primit_surf)->hf_options->fd_struct->filter_level;
		gtk_container_add(GTK_CONTAINER(hfw->creation_container),
			hfw->hf_options->surfadd_dialog);
	}
	if (hfw->hf_options->current_dialog)
		gtk_widget_hide(hfw->hf_options->current_dialog);
	gtk_widget_show(hfw->hf_options->surfadd_dialog);
	hfw->hf_options->last_creation_action = _("SurfSum");
	hfw->hf_options->current_dialog = hfw->hf_options->surfadd_dialog;
	hfw->hf_options->current_calculation = (gpointer) surfadd;
	hfw->hf_options->current_callb = hfsurfadd_callb;
	hfw->hf_options->current_options = (gpointer) hfw->hf_options->surfadd_options;

	gener_hf(hfw);
}
